## Get Presentation Tarball

include(intro/slides.md)

<!--

## Get Presentation Tarball

If you haven't got the presentation tarball, yet, now is the time to download
it. It will come in very handy because it contains all the commands you'll need
for this part in an easily pastable format. So if you haven't got it, yet,
please download it now.

-->

## Pre-checks for deploying a cluster using magnum

* On the Controller node

      source .openrc

      magnum service-list

      openstack image show openstack-magnum-k8s-image


* Package for the magnum image

      zypper in openstack-magnum-k8s-image-x86_64

* Create the glance image if required

* [cmd/create-glance-image.sh]

```
include(cmd/create-glance-image.sh)
```

<!--
## Pre-checks for deploying a cluster using magnum

Login to the controller node and source the openstack credentials. Check if the
magnum services are up and running. At least one magnum-conductor service
should be up and running. The API is not a service under its definition of
service, so service-list will not show it.

As mentioned earlier, we will be using the SLE12SP2 image for our demo. We will
always have this image created if we use the magnum barclamp for deployment as
it is a part of the magnum cookbook.  When it is not we can install the
openstack-magnum-k8s-package which will have the image and then upload it to
glance. The important bit of the image creation is the os-distro argument which
needs to be present when creating the image because when magnum builds the
cluster-template it searches for the os_distro field. If it is not found it
will throw an error. `ERROR: Image doesn't contain os-distro field. (HTTP 404)`

-->

## Pre-checks for deploying a cluster using magnum (cont.)

      openstack keypair show default
      openstack flavor show m1.magnum
      openstack network show floating

* [cmd/create-keypair.sh]

```
include(cmd/create-keypair.sh)
```

* [cmd/create-flavor.sh]

```
include(cmd/create-flavor.sh)
```
<!--

## Pre-checks for deploying a cluster using magnum (cont.)

The keypair should already be present on our mkcloud created cloud.  If not we
can upload the ssh public key using the `openstack keypair create` command.

Then we can check if the favor we need exists. It should be there as it is a part
of the magnum recipe, otherwise you can create it using the `openstack flavor create`
command.

We also need to check if the external network is created.
In our deployment the name will be floating. This network basically allows access
to the outside for the cloud.

-->

## Commands we will use frequently

    magnum cluster-template-list
    magnum cluster-template-show k8s_template
    magnum cluster-list
    magnum cluster-show k8s_cluster

<!--

## Commands we will use frequently

These are some of the commands that we will be using very often here on.

 * magnum cluster-template-list
 * magnum cluster-template-show k8s_template
 * magnum cluster-list
 * magnum cluster-show k8s_cluster

This will list the template, show details of the cluster-template, list the
cluster and show details about the cluster. Please remember this slide for
future reference.

-->

## Steps to deploy a Kubernetes cluster using magnum CLI

* Create the cluster template

* [cmd/create-cluster-template.sh]

```
include(cmd/create-cluster-template.sh)
```

<!--

## Steps to deploy a Kubernetes cluster using magnum CLI

First of all, we create the cluster template.  We need to specify the name for
the template, the magnum image name, the keypair, the external network to be
used, dns-nameserver, flavors for the master and minion nodes, docker volume
size, the network driver and the container orchestration engine which is docker
in our case but it can be Swarm or Mesos. To diable ssl support pass
`--tls-disabled`. You can also pass `--master-lb-enabled` to enable load
balancing for etcd and the kubernetes API. We can simply run `magnum cluster-template-list`
to see if the template is created and then run `magnum cluster-template-show`
to see the details of the cluster.

It is important to specify the --external-network since this assigns floating ips to the
nodes which allows access to the internet. If this is not setup properly then the etcd discovery
will fail(if using public discovery) and the container images cannot be downloaded, among other
failures.

-->


## Interlude: `cluster-template-show` Output

```
include(output/cluster-template-show)
```

## Steps to deploy a Kubernetes cluster using magnum CLI(cont.)

* Create the cluster

* [cmd/create-cluster.sh]

```
include(cmd/create-cluster.sh)
```
<!--

## Steps to deploy a Kubernetes cluster using magnum CLI(cont.)

Then we create the cluster by specifying the number of nodes and the template
to be used (which is the one we just created). This will take some time as this
only starts the process, and in the background Heat will now begin provisioning
of the nova instances, and trigger cloud-init to configure all the things
inside them. We can run the `magnum cluster-list` command here to see the
the current status of the cluster.

-->

## Interlude: `cluster-show` Output

```
include(output/cluster-show)
```


## Monitoring and updating the kubernetes cluster


* List Heat stacks

```
openstack stack list
```

* Get cluster's stack ID:

* [cmd/stack-uuid.sh]

```
include(cmd/stack-uuid.sh)
```

* Monitor stack status

```
watch openstack stack resource list -n 5 $stack \| grep -v CREATE_COMPLETE
```

* Update the cluster

* [cmd/update-cluster.sh]

```
include(cmd/update-cluster.sh)
```

<!--

## Monitoring and updating the kubernetes cluster

We can now run the `openstack stack list` command to see the heat stack
building.

In order to monitor our cluster's stack, we can watch the stack's resource list
to see when the stack creation is complete:

```
include(cmd/stack-uuid.sh)
watch openstack stack resource list -n 5 $stack \| grep -v CREATE_COMPLETE
```

Once it is complete we can run `magnum cluster-show k8s_cluster` to see the
details of the cluster like the master and minion address and the node count.

At any later stage we can update the cluster by using the `magnum cluster-update`
command. We can add more nodes to the cluster at any point with this command.
Currently the update command rebuilds all the nodes in the cluster so you will
ultimately end up creating a new cluster. You need to be specifically careful
while using this with customers deployments because if the cluster has container
workload deployed you will loose them all if you update the cluster and this can
cause serious problems.

This might be too big for our standard mkcloud Magnum setup which has this
configuration

```
export nodenumber=3
export nodenumbercompute=2
```

But can work with a setup with a bigger setups with configuration like

```
export nodenumber=4
export nodenumbercompute=3
```

-->

## Accessing the Kubernetes cluster using magnum CLI


* Package needed: openstack/python-magnumclient

* Generate and export the kubernetes cluster config to use it from anywhere


      magnum cluster-config --dir $HOME k8s_cluster

      export KUBECONFIG=$HOME/config

* Install kubectl (https://kubernetes.io/docs/tasks/tools/install-kubectl/)

* [cmd/install-kubectl.sh]


```
include(cmd/install-kubectl.sh)
```

<!--

## Accessing the Kubernetes cluster using magnum CLI

Next we need to access our cluster. There are two ways to do that.  Either by
running `magnum cluster-show` and getting the IP of the master node and SSHing
to, it or accessing it from any machine using the magnum CLI.  If you ssh to
the master node then you can directly run the kubernetes commands as
kubectl(the command line interface for kubernetes) will be installed and
configured by cloud init.

For the second method, we will need python-magnumclient installed.  Then you
can get the config of your specific cluster using the `magnum cluster-config`
command. This will create a config file, which it does not overwrite by
default so you need to be careful where you are running it and delete or move
any old config file that might still exist.

You also need to be careful about making sure the config file is up-to-date as
it can happen over time that the IP for the master node changes and needs to be
updated in the config file. If you want to be on the safe side you just run
`magnum create-config` once again.  Next you would need to install kubectl.
Unfortunately we do not have a package for that in SLES, yet. For now you can
either run the following:

```
include(cmd/install-kubectl.sh)
```

or visit the [offical kubernetes page](https://kubernetes.io/docs/tasks/tools/install-kubectl/) to download it.

-->

## Kubernetes commands we will frequently use


**NOTE: Can be run directly on the master node or from anywhere by exporting the config like above**

      kubectl version

      kubectl cluster-info dump

      kubectl explain <element>

      kubectl get nodes

      kubectl get pods

      kubectl get svc

<!--

## Kubernetes commands we will frequently use

These are some commands we will use often going ahead.

* `kubectl version`: This will give you two versions: a client and server
  version. The server is running on the cluster's Kubernetes master node and
  kubectl is the client.

* `kubectl cluster-info dump`: Gives you verbose information about the cluster.

* `kubectl explain <element>`: This provides an detailed documentation of the
  manifest components.

* `kubectl get <element>`: The get command can fetch information about any
  resource of a Kubernetes cluster.

-->


## Running our first pod

* Create pod: `kubectl create -f nginx.yaml`

* Create a file (e.g nginx.yaml) describing a pod running nginx:

* [cmd/nginx.yaml]

```
include(cmd/nginx.yaml)
```

<!--

## Running our first pod

Pods are the most basic unit in a kubernetes cluster. It is a collection
of one or more containers which are always co-located, co-scheduled and
run in s shared context.
The yaml files are called the manifests and can also be in json format.

Here's what we will use:

```
include(cmd/nginx.yaml)
```

Without delving into much detail what you see in this file is

* `apiVersion`: the value of this has to be in the list of versions given by
  the api-version command
* `kind`: specifies the kind of resource we are defining
* `metadata`: a key-value list
* `spec`: the specifications of the pod. Here we specify that we want to create
  a pod named nginx, the name of the container is `nginx` and the
  `containerPort` is 80.

Now we create the pod using the create command:

```
kubectl create -f nginx.yaml
```

-->

## Running our first service

* Create service: `kubectl create -f nginx-service.yaml`

* Create a file (e.g nginx-service.yaml) describing a service for the nginx pod:

* [cmd/nginx-service.yaml]

```
include(cmd/nginx-service.yaml)
```

<!--

## Running our first service

Next we create a service. One of the most powerful abstractions in Kubernetes
is the service resource. While pods can come and go, services are an
abstraction that makes guarantees to the consumer about a specific service. The
idea is that the consumer should only care about the service being provided,
rather than the underlying implemetation.

Services can be made available to external clients or they can only be used for
internal purpose.

Most of the information in the file is the same as for the pod, but there are
some differences:

* In the `spec` section we specify a port mapping where we specify that the
  host's port 80 maps to the pod's port 80.

* The `selector` section will inform kubernetes that this service is provided
  by pods with label "app=nginx"

Now we create the service using the create command.

-->

# Work in progress

## Making the service available to the internet

* Enabling external loadbalancer feature

  * [Upstream Patch](https://review.openstack.org/#/c/456628/)


* Login to master node and modify /etc/sysconfig/kubernetes_openstack_config
  to add username and password

* Restart service


      systemctl restart kube-controller-manager


## Running modified service

* Create service: `kubectl create -f nginx-service1.yaml`

* Create a file (e.g nginx-service1.yaml) describing a service for the nginx pod:

* [cmd/nginx-service1.yaml]

```
include(cmd/nginx-service1.yaml)
```

# Demo: Deploying kubernetes cluster from horizon

<!--

## Demo: Deploying kubernetes cluster from horizon

First we will deploy magnum by applying the magnum barclamp using the crowbar
UI on the node you want, mostly likely the dashboard node. Then we switch to
the openstack dashboard. We will use the SLES12SP2 based Magnum image to
deploy the kubernetes cluster.

First we need to create a cluster template which is a set of parameters that
specify how to deploy our kubernetes (or any other cloud orchestration engine)
to Magnum. Most of the metadata describing a cluster are set in the
ClusterTemplate it is associated with.

We need to choose the image, master flavor, minion flavor, backend and volume
for container data. For networking for kubernetes we use flannel as the network
driver. We only need to specify the external network name for the Floating IPs.
We will use the local docker registry.

After the cluster template we create the cluster, where we only need to specify
the number of the master and minion nodes. This starts the provisioning process
to deploy the kubernets cluster. Magnum uses heat templates, so our deployment
can be seen in the stack topology tab. Heat spawns the nova instances for the
master and minion nodes. We can check on the console to see how the master node
is booting and which services are configured and started. The cloud-init
service will take the metadata and configure the rest of the kubernetes
services needed. It deploys the services needed on the Kubernetes master first
like etcd, kube-master, kube-scheduler and apicontroller which will generate
ssh keys for the node.

Once the master is finished cloud-init will run on the minion (or minions)
where the containers are started. It will create the minions using the flavor
we specified and customize everything inside. On the network topology we can
see all the instances are in the same network, connected together and routed to
the outside.

-->

include(handson/troubleshooting.md)
