# Troubleshooting

<!--

# Troubleshooting

So much for how Magnum looks in good working condition. Now we are coming to
the part where we will deliberately break Magnum in various common ways and try
to fix it again.

-->

## Preparation: Some Sabotage

* No Heat API access for floating IPs

* [cmd/iptables-break.sh]

```
include(cmd/iptables-break.sh)
```

<!--

## Preparation: Some Sabotage

Before we begin we'll need to create two packet filter rules to ensure things
go South later. You will find these roles as a ready-to-run shell script in
`cmd/iptables-break.sh`:

```
include(cmd/iptables-break.sh)
```

They will prevent access to the Heat API from the floating IP network while
still allowing access from the controller node itself. Please just create them
for now and move on. We'll provide a detailed explanation of how and why later.
For now we'll need to move on because we'll need to recreate our Kubernetes
cluster in order for it to suffer from this act of sabotage and this will take
time. So please create the rules now, we'll take care of rebuilding the cluster
on the next slide.

-->

## Preparation: Create a Fresh Cluster

* Delete existing cluster

* [cmd/delete-cluster.sh]

```
include(cmd/delete-cluster.sh)
```


* Create a new cluster

* [cmd/create-cluster.sh]

```
include(cmd/create-cluster.sh)
```

<!--

## Preparation: Create a Fresh Cluster

Ok, now that everyone's got the iptables rules in place, let's rebuild the
cluster so it is finished failing by the time we are ready to analyze the
error:

### Delete Existing Cluster

```
include(cmd/delete-cluster.sh)
```

### Create the New Cluster

```
include(cmd/create-cluster.sh)
```

-->

## When is a Cluster Complete?

* Signalling mechanism: Heat [WaitCondition](https://docs.openstack.org/developer/heat/template_guide/openstack.html#OS::Heat::WaitCondition)

  * One for each instance

  * Instance curls a magic Heat API URL after `cloud-init` completes

* Possible failure modes:

  * Heat API unreachable

  * cloud-init fails somewhere along the way

  * Cluster takes longer than WaitCondition timeout to complete

<!--

## When is a Cluster Complete?

Before we get started on troubleshooting clusters, let's take a look at what
constitutes a completed cluster. In order to indicate completion, Magnum uses
Heat's [WaitCondition](https://docs.openstack.org/developer/heat/template_guide/openstack.html#OS::Heat::WaitCondition) mechanism. A WaitCondition gives 
you a magic Heat API URL you can insert into a cloud-init user data script to
signal to Heat it has reached the point where the URL is curled. Heat will then
set the WaitCondition associated with that URL to CREATE_COMPLETE.

Magnum creates such a WaitCondition along with every instance. Its curl command
is triggered at the very end of the user data payload, so if cloud-init manages
to get to that point, this indicates a successfully deployed instance. If all
of the stack's WaitConditions transition to state CREATE_COMPLETE that way, the
cluster itself is marked complete.

There are a number of ways this can go wrong, and they are the main failure
modes for cluster creation (barring problems with resource creation such as the
ever popular "no valid host was found" from Nova). First of all, the instance
may be unable to reach the Heat API for some reason. We are going to use the
blunt instrument of the iptables rule we created to bring that situation about.

Second, the instance may simply take to long, causing the wait condition to
timeout and thus transition to CREATE_FAILED.

Third, some part of the deployment process may fail, causing cloud-init never
to reach the point where it curls the Heat API.

-->


## 1: Heat API Unreachable / Timeout

* Step 1: obtain cluster's main Heat stack UUID

* [cmd/stack-uuid.sh]

```
include(cmd/stack-uuid.sh)
```

* Step 2: find failing WaitCondition's [nested](https://docs.openstack.org/developer/heat/template_guide/composition.html) stack name

* [cmd/nested-name.sh]

```
include(cmd/nested-name.sh)
```

<!--

## 1: Heat API Unreachable / Timeout

By now we should all have a cluster in state CREATE_FAILED. So let's triage it
and find out which of the failure modes we are dealing with. The first thing we
check for is a wait condition timeout or unreachable Heat API. You can tell
these two apart fairly easily.

First of all, we need to figure out the cluster's main Heat stack UUID. We can
get this UUID from a `magnum cluster-show`:

```
include(cmd/stack-uuid.sh)
```
-->


## Interlude: `cluster-show` Output

```
include(output/cluster-show-timeout)
```

<!--

That command is a bit opaque. We did test it and it works, but we'd like to
show you what it works on:

```
include(output/cluster-show-timeout)
```

It looks for the line with the `stack_id` property and extracts the second
column which contains the UUID for the cluster's main Heat stack.

-->

## 1: Heat API Unreachable / Timeout

* Step 1: obtain cluster's main Heat stack UUID

* [cmd/stack-uuid.sh]

```
include(cmd/stack-uuid.sh)
```

* Step 2: find failing WaitCondition's [nested](https://docs.openstack.org/developer/heat/template_guide/composition.html) stack name

* [cmd/nested-name.sh]

```
include(cmd/nested-name.sh)
```

<!--

Next we'll list this stack's resources and look for the failing WaitCondition:

```
include(cmd/nested-name.sh)
```

The failing WaitCondition will be in a [nested](https://docs.openstack.org/developer/heat/template_guide/composition.html)
stack. "nested" means a Heat 
stack created from inside a Heat stack in this context. Magnum creates a main
Heat stack with one or more nested stacks for the Kubernetes masters and
Kubernetes minions. How many there are depends on the `--num-nodes` and
`--num-masters` options. 

-->

## Interlude: `resource list` Output

```
include(output/resource-list-timeout)
```

<!--

Again, this command is a bit opaque, so let's look at what it operates on again:

```
include(output/resource-list-timeout)
```

In that list we look for a `OS::Heat::WaitCondition` resource in state
`CREATE_FAILED` (we will only have one in the case at hand) Once we've found it
we look at the 4th column, which contains the name of the nested Heat stack the
WaitCondition's is defined in. Now that we've got it, we will use that stack
name in the next step to figure out whether anything has tried to signal the
wait condition.

-->

## 1: Heat API Unreachable / Timeout (cont.)

* Step 3: check for wait condition signalling in log

* [cmd/signal-log.sh]:

  ```
include(cmd/signal-log.sh)
  ```

  * If this yields results but the WaitCondition has failed, deployment was
    successful, but the WaitCondition already transitioned to `CREATE_FAILED`
    because it took too long.

  * If this does not yield any results, we'll need to keep looking...

<!--

The magic signalling URL for a wait condition always ends in `/signal` and
contains the stack name of the Heat stack it was defined in. Hence we grep for
this combination in the Heat API's log file:

```
include(cmd/signal-log.sh)
```

Typically you will be dealing with an HA enabled cloud and thus multiple
heat-api instances, so you'll have to perform this step for *all* heat-api log
files to be sure.

If this search yields a result with a `200` response but the WaitCondition is
in `CREATE_FAILED` state nonetheless, that indicates a genuine timeout. The
`200` status is a bit misleading since wait condition signalling will report
success back to whoever issues a request against that magic URL, but won't
change the WaitCondition's state once it has timed out. We will later bring
this situation about as well and show you how to fix the problem.

In the case at hand we will not find anything, and can thus rule out late
signalling as the problem. The remaining possiblities are connectivity issues
and something going wrong in one of the user-data scripts. Since we lovingly
handcrafted the network problem leading to this situation we already know what
went wrong. Before we fix it let's take a quick look at how it manifests inside
the affected instance because, though. For this particular network problem will
manifest as a user-data script failure inside the instance and we'd like to
show you how to spot and debug that.

-->

## 2: Failing user-data script

* Step 1: obtain failing cluster's main Heat stack UUID 

* [cmd/stack-uuid.sh]

```
include(cmd/stack-uuid.sh)
```

* Step 2: obtain nested stack name for failing WaitCondition

* [cmd/nested-name.sh]

```
include(cmd/nested-name.sh)
```

* Step 3: obtain floating IP address for instance with failed WaitCondition

* [cmd/show-floating.sh]:

```
include(cmd/show-floating.sh)
```

<!--

## 2: Failing user-data script

To debug a failing user data script we will have to log in to the instance
where the user-data scripts failed, causing the WaitCondition not to trigger.
To that end we need to figure out a few things again.

First of all we need to obtain the failing cluster's main Heat stack UUID
again:

```
include(cmd/stack-uuid.sh)
```

Once we've got it, we will again look at that stack's resource list and find
the stack name of the nested stack the failing WaitCondition is part of:

```
include(cmd/nested-name.sh)
```

With that stack name we can now obtain the problematic node's floating IP
address:

```
include(cmd/show-floating.sh)
```

Depending on whether the WaitCondition failed for a Kubernetes master or a
Kubernetes minion, the output name will differ. This is why I listed two
commands here. In our case we'll use the first command because the Minion
stacks are only created once all WaitConditions in the Master stacks have
transition to `CREATE_COMPLETE`. Since our little act of sabotage threw a
spanner into the works for that first step already, we will be dealing with a
failed Kubernetes master now, so that's the machine we log in to.

-->

## 2: Failing user-data script (cont.)

* Step 4: SSH login (as root) to floating IP address

* Step 5: examine `/var/log/cloud-init-output.log`

```
2017-05-12 09:31:09,517 - util.py[WARNING]: Failed running /var/lib/cloud/instance/scripts/part-007
```

* Step 6 (optional): locate source script under `/usr/lib/python2.7/magnum/`,
  find problem, add debug output, fix script...

<!--

Now that you have the floating IP address, please log in to the machine where
cloud-init hiccupped and look at `/var/log/cloud-init-output.log` (do not
bother with `/var/log/cloud-init.log` - it's always empty). That will tell us
which script failed and give us its output. In this case it's quite
straightforward: the spanner we threw into the works earlier caused a
"Connection Refused" on this side.

Sometimes it's not quite so straightforward, though. In that case you might
look at the script on the instance, or the source script that comes with the
`magnum` python module. You might even add debugging output to it, if the
scripts regular output is not enough.

This is a bit beyond the scope of this workshop, though: we'll be quite happy
if you debug the problem up to this point and then provide a bug report with
the contents of `/var/log/cloud-init-output.log` and steps to reproduce the
problem. If you are interested we might be able to take a look at how to add
debugging output and manipulate scripts after we're through with the slides.
First we'll need to finish with the slides, though, for we've got two more
failure modes for you.

-->

## Interlude: Fix iptables, Break Timeout...

* Fix iptables

* [cmd/iptables-unbreak.sh]

```
include(cmd/iptables-unbreak.sh)
```

* Break WaitCondition timeout

* [cmd/break-timeout.sh]

```
include(cmd/break-timeout.sh)
```

<!--

## Interlude: Fix iptables, Break Timeout...

Before we go on I'll need you to quickly remove the iptables rule we used to
sabotage WaitCondition signalling...

```
include(cmd/iptables-unbreak.sh)
```

...and break the WaitCondition timeout instead. More on this in a bit.


```
include(cmd/break-timeout.sh)
```

We only need to remove the reject role here because the ACCEPT one doesn't do
any harm.

-->

## Interlude: ...and Rebuild Cluster

* Delete existing cluster

* [cmd/delete-cluster.sh]

```
include(cmd/delete-cluster.sh)
```

* Create new cluster

* [cmd/create-cluster.sh]

```
include(cmd/create-cluster.sh)
```

<!--

## Interlude: ...and Rebuild Cluster

And now we'll rebuild the cluster without these rules in place, so delete the
old cluster first...

```
include(cmd/delete-cluster.sh)
```

...and recreate it:

```
include(cmd/create-cluster.sh)
```

Once everybody has created the new cluster, let's wait a minute and check the
cluster's state again using `magnum cluster-show`.

-->

## 3: Genuine Timeout

* Step 1: obtain cluster's main Heat stack UUID

* [cmd/stack-uuid.sh]:

```
include(cmd/stack-uuid.sh)
```

* Step 2: find failing WaitCondition's nested stack name

* [cmd/nested-name.sh]:

```
include(cmd/nested-name.sh)
```

* Step 3: check for wait condition signalling in log

* [cmd/signal-log.sh]:

  ```
include(cmd/signal-log.sh)
  ```

<!--

## 3: Genuine Timeout

Everybody should have a cluster in `CREATE_FAILED` state now. So lets do the
same thing we did in the beginning to determine whether we were dealing with a
genuine timeout or a user-data script/network failure.

First, we obtain the cluster's Heat stack UUID again:

```
include(cmd/stack-uuid.sh)
```

Next, we go through the main stack's resource list to find the name of the
nested stack where the failing WaitCondition resides:


```
include(cmd/nested-name.sh)
```

Finally we grep for the wait condition's URL components in `/var/log/heat/heat-api.log` 
again. If we gave it enough time we to let the Kubernetes master's cloud-init
scripts to finnish, we will find an attempt to signal the WaitCondition, even
with a 200 OK. That is misleading, though: since the WaitCondition was already
in state CREATE_FAILED at this point in time, it's state didn't change anymore.

This sort of problem can easily be fixed by adjusting the cluster's timeout
upwards. There's no hard and fast rule for how much to adjust: that depends on
the cloud's load and general slowness in deploying resources. Just deploy a
couple of clusters and use the longest deployment time with a few minutes'
safety margin added on top. Our current default timeout is fairly tight,
unfortunately, so you may encounter this problem a lot, but we've got a patch
on the way to raise it.

-->

## Interlude: Fix Timeout, Break etcd

* Restore old WaitCondition Timeout

* [cmd/fix-timeout.sh]

```
include(cmd/fix-timeout.sh)
```

* Add a rogue DNS entry for `discovery.etcd.io`

* [cmd/break-etcd.sh]

```
include(cmd/break-etcd.sh)
```

<!--

Before we go on with the next error, please fix the timeout once more so we can
properly provoke the next error. While we'll hit it anyway, even with an
undersized timeout, it's cleaner to fix this first.

Next, add a bogus entry for `discovery.etcd.io` to `/etc/hosts`:

```
include(cmd/break-etcd.sh)
```

That is all there is to our next act of sabotage.

-->

## Redeploy Cluster Once More

* Delete existing cluster

* [cmd/delete-cluster.sh]

```
include(cmd/delete-cluster.sh)
```


* Create a new cluster

* [cmd/create-cluster.sh]

```
include(cmd/create-cluster.sh)
```

<!--

## Redeploy Cluster Once More

With `discovery.etcd.io` freshly unreachable, redeploy your cluster now:

### Delete existing cluster

```
include(cmd/delete-cluster.sh)
```

### Create a new cluster

```
include(cmd/create-cluster.sh)
```

-->

## Failure to Get Discovery URL

```
include(output/cluster-show-discovery)
```

<!--

## Failure to Get Discovery URL

So let's see how our cluster is faring:

```
include(output/cluster-show-discovery)
```

Oops. That failed fast. What Magnum tried to do here was to obtain a discovery
URL. This URL is randomly generated by `discovery.etcd.io`. If that had been
successful, Magnum would have passed the discovery URL to all cluster members,
who would have used it to register as the cluster's etcd cluster members.
Without a discovery URL this is not possible.

So what do we do about this?

-->

## Static etcd Cluster to the Rescue

* We have various Provisions for offline (Cloud without Internet access) operation.
  
  * Only in OpenSUSE Kubernetes driver

  * Includes `etcd` operation without discovery

  * Based on static list of cluster members

* Caveat: cluster update is effectively rebuild of _all_ cluster nodes

<!--

## Static etcd Cluster to the Rescue

Since some of our customers have clouds without access to the Internet (or only
heavily restricted access), we came up with various mechanisms for allowing
Magnum to deploy a cluster in this sort of environment. Normally it is a bit
hipster and assumes that it can reach various third-party services such as the
public `etcd` registry or Docker Hub. We came up with local solutions for the
things these services are used for to allow Magnum to function without Internet
access. These mechanisms are only available in our OpenSUSE Magnum driver, so
clusters on Fedora, CoreOS or Ubuntu images will continue to require Internet
access.

One of the things we came up with as part of these efforts is discoveryless
`etcd` operation. Rather than register cluster membership through a discovery URL
obtained from , every instance in the etcd cluster gets a list of all other instance's
IP addresses.

This will always work but it causes a bit of a problem when updating clusters:
since this list is recomputed every time an instance is added or removed, and
then inserted into _every_ instance's user data payload, all these instances
will be rebuilt (in order to cause the updated user data payload to run). So
if someone assumes they can simply add nodes to a cluster they have active
workloads on, they are going to lose these workloads if they actually perform a
`cluster-update` operation.

-->

## Rebuild Cluster Without Discovery URL

* Delete existing cluster

* [cmd/delete-cluster.sh]

```
include(cmd/delete-cluster.sh)
```


* Create a new cluster

* [cmd/create-cluster-discovery.sh]

```
include(cmd/create-cluster.sh)
```

## Rebuild Cluster Without Discovery URL

* Delete existing cluster

* [cmd/delete-cluster.sh]

```
include(cmd/delete-cluster.sh)
```


* Create a new cluster

* [cmd/create-cluster-discovery.sh]

```
include(cmd/create-cluster-discovery.sh)
```


<!--

## Redeploy Without Discovery URL

Let's redeploy our cluster with `--discovery-url none` now.

### Delete existing cluster

```
include(cmd/delete-cluster.sh)
```

### Create a new cluster

```
include(cmd/create-cluster-discovery.sh)
```

-->

## Undo etcd Sabotage

* Remove `/etc/hosts` entry

* [cmd/fix-etcd.sh]

```
include(cmd/fix-etcd.sh)
```

* More `etcd` failure modes: https://docs.openstack.org/developer/magnum/troubleshooting-guide.html#etcd-service

<!--

## Undo etcd Sabotage

As usual, please undo the etcd sabotage:

```
include(cmd/fix-etcd.sh)
```

Also, there are various other ways for etcd to break. For instance, Magnum may
be able to request a discovery URL but it may then become unreachable at some
much later point. Upstream covers most of these problems in their [Troubleshooting Guide](https://docs.openstack.org/developer/magnum/troubleshooting-guide.html#etcd-service).

Since most of the solutions in there revolve around simply restarting `etcd`
and `flannel` until the cluster is back in working order, I'll spare you to
tedium. Also, this is hard to sabotage reliably. Just remember that upstream
troubleshooting guide for when you encounter this kind of problem.

-->

## Bonus Error: cluster_user_trust

* Patch for [CVE-2016-7404](http://lists.suse.com/pipermail/sle-security-updates/2017-May/002853.html)
  breaks clusters with `registry-enabled` set in ClusterTemplate

* Relevant for [Magnum offline scenario](https://www.suse.com/documentation/suse-openstack-cloud-7/book_cloud_suppl/data/sec_deploy_kubernetes_without.html) (cloud without Internet access)

<!--

Finally we have a bonus error condition for you. This one does not require any
sabotage since our default configuration triggers it. It is also fairly easy to
debug and fix, but let me give you a quick rundown on its history before we get
to the actual debugging part:

We recently fixed a bit of a security issue in Magnum. Before our patch, Magnum
would always pass Keystone credentials that allowed full Openstack API access
in the project the cluster was created in, as the creating user and with all
that user's roles in the project. That's a lot of out-of-band access, especially 
if the admin user is foolish enough to create a cluster in the admin tenant
(that would give somebody able to compromise an instance the keys to the
kingdom).

Our patch modified Magnum not to require these credentials in most cases, but
there are two things that continue to use Openstack APIs from inside instances:
the local docker registry which stores its images in Glance and the rexray
Docker volume driver which uses Cinder as its storage backend. If either is
used, we need to revert to the old, insecure behaviour. For us this is mostly
relevant in our offline scenario since that requires `registry_enabled` to be
true in the ClusterTemplate.

-->

## cluster_user_trust: Create Template

* Create ClusterTemplate with `--registry-enabled`

* [cmd/create-template-registry.sh]

```
include(cmd/create-cluster-template.sh)
```


## cluster_user_trust: Create Template

* Create ClusterTemplate with `--registry-enabled`

* [cmd/create-template-registry.sh]

```
include(cmd/create-template-registry.sh)
```

<!--

The key to this little act of sabotage is using a ClusterTemplate with
registry_enabled=true, so let's create one:

```
include(cmd/create-template-registry.sh)
```

-->

## cluster_user_trust: Recreate cluster

* Delete old cluster

* [cmd/delete-cluster.sh]

```
include(cmd/delete-cluster.sh)
```

* Create cluster with new template

* [cmd/create-cluster-registry.sh]

```
include(cmd/create-cluster.sh)
```

## cluster_user_trust: Recreate cluster

* Delete old cluster

* [cmd/delete-cluster.sh]

```
include(cmd/delete-cluster.sh)
```

* Create cluster with new template

* [cmd/create-cluster-registry.sh]

```
include(cmd/create-cluster-registry.sh)
```

<!--

Now let's get rid of the old cluster...

```
include(cmd/delete-cluster.sh)
```

...and create a new one using our shiny new template:

```
include(cmd/create-cluster-registry.sh)
```

And now let's do another `magnum cluster-show` on it...

-->

## Error: cluster_user_trust=false

* Error message in `status_reason:

```
   This cluster can only be created with
   trust/cluster_user_trust = True in magnum.conf
```

* Fix

  * Set `trustee["cluster_user_trust"] = True` (raw mode setting in Crowbar)

  * Run each cluster in a dedicated project with tight quotas to limit attack surface

<!--

## Error: cluster_user_trust=false

Oops, that didn't go too well. So what happened here?

If we attempt create a cluster with registry-enabled set in its template on
Cloud 7 with default settings, we will get this error message:

```
This cluster can only be created with trust/cluster_user_trust = True in magnum.conf
```

To make the offline scenario work in this case you will need to set the raw
mode setting `trustee["cluster_user_trust"]` to `True` in Crowbar. This
restores the old, insecure behaviour for clusters with `registry-enabled` or
`volume_driver=Rexray`. With this configuration change in place you will be
able to create the cluster now.

It is advisable to run each cluster with `registry-enabled` in a project of its
own with all quotas set to exactly match the cluster's resource consumption.
This way, an attacker who manages to gain access to the cluster can at worst
destroy the cluster itself but will neither be able to affect other projects'
OpenStack resources nor will they be able to create resources beyond those
allocated for the cluster itself.

-->
